import sys
import logging
import yaml
import os
import time

from tomography.omps_tomography import OMPSTomography
import tomography.omps_processing.omps_jobs_database as omps_sql
from tomography.omps_processing.omps_util import convert_pickle_to_l2

from random import randint
from time import sleep


def omps_retrieval(yaml_file):
    sleep(randint(1, 10))
    start_time = time.time()
    with open(yaml_file) as f:
        settings = yaml.load(f)

    try:
        total_jobtime_hours = settings['westgrid settings']['jobtime_hours']
    except:
        total_jobtime_hours = None

    try:
        one_job_time = settings['westgrid settings']['one_job_hours'] * 3600
    except:
        one_job_time = None
        
    # setup logging format
    if settings['logging settings']['save_log_output']:
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    finished = False
    jobs = omps_sql.OMPSTomographyJobs(settings['database settings']['database_name'],
                                       settings['database settings']['table_name'],
                                       settings['database settings']['database_folder'])

    if settings['logging settings']['save_log_output']:
        log_file = settings['logging settings']['log_folder'] + settings['logging settings']['log_file']

    # get the first job to process
    orbit = jobs.get_next_entry()
    jobs.set_entry_as_processing(orbit)
    if orbit is None:
        return

    # Have processing to do.  Start by checking if folder exist, if not create them
    outfolder = settings['directory settings']['output_directory']
    outfolder_l2 = os.path.abspath(outfolder + r'/L2/')
    outfolder_raw = os.path.abspath(outfolder + r'/raw/')
    if not os.path.exists(outfolder_l2):
        os.makedirs(outfolder_l2)
    if settings['debug options']['save_raw_output']:
        if not os.path.exists(outfolder_raw):
            os.makedirs(outfolder_raw)

    if settings['logging settings']['save_log_output']:
        logfolder = settings['logging settings']['log_folder']
        if not os.path.exists(logfolder):
            os.makedirs(logfolder)

    while not finished:
        # setup/change log file
        if settings['logging settings']['save_log_output']:
            fileh = logging.FileHandler( log_file + str(orbit) + '.log')
            fileh.setFormatter(formatter)
            log = logging.getLogger()
            for hdlr in log.handlers:
                log.removeHandler(hdlr)
            log.addHandler(fileh)
            log.info('\n#############################################################\n'
                     'PROCESSING: orbit: ' + str(orbit) +
                     '\n#############################################################\n')

        try:
            omps = OMPSTomography(orbit)

            for key in settings['retrieval settings']:
                omps.ret_opt[key] = settings['retrieval settings'][key]
            for key in settings['radtran settings']:
                omps.ret_opt[key] = settings['radtran settings'][key]
            omps.ret_opt['omps_l1_folder'] = settings['directory settings']['omps_l1_folder']
            omps.ret_opt['omps_l1_anc_folder'] = settings['directory settings']['omps_l1_anc_folder']

            output = omps.retrieve_orbit()
            if settings['debug options']['save_raw_output']:
                raw_file = outfolder_raw + '/RetInfo_' + str(orbit) + '.h5'
            else:
                raw_file = None
            save_averaging_kernel = settings['debug options']['save_averaging_kernel']
            convert_pickle_to_l2(output, outfolder_l2 + '/o' + str(orbit) + '.h5', raw_file, save_averaging_kernel=save_averaging_kernel)
            jobs.set_entry_as_finished(orbit, failed=0)
        except Exception as e:
            if settings['logging settings']['save_log_output']:
                log.error('Orbit failed to process:', exc_info=True)
            jobs.set_entry_as_finished(orbit, failed=1)

        if (total_jobtime_hours is not None) and (one_job_time is not None):
            time_left = total_jobtime_hours*3600 - (time.time() - start_time)
            if time_left < one_job_time*1.1:
                finished = True

        orbit = jobs.get_next_entry()
        if orbit is None:
            finished = True
        else:
            jobs.set_entry_as_processing(orbit)

if __name__ == "__main__":
    print(sys.argv[1])
    omps_retrieval(sys.argv[1])
