import matplotlib.pyplot as plt
import numpy as np
import pickle
import scipy.optimize as opt
import h5py


def gauss(x, p):
    return 1.0/(p[1]*np.sqrt(2*np.pi))*np.exp(-(x-p[0])**2/(2*p[1]**2))


def make_averaging_kernel_plots(A):
    LOC_IDX = 80
    NUM_ALT = 55
    plot_alts = slice(12, 40)

    plt.style.use('ggplot')

    alts = np.linspace(0.5, NUM_ALT - 0.5, NUM_ALT)
    plt.figure()
    # Vertical averaging kernels
    data = [np.sum(A[(LOC_IDX*NUM_ALT):(LOC_IDX+1)*NUM_ALT, idx::NUM_ALT], axis=1) for idx in range(NUM_ALT)]
    plt.plot(np.array(data)[:, plot_alts], np.linspace(0.5, NUM_ALT - 0.5, NUM_ALT))
    plt.ylim(0, 60)

    integrated = np.sum(np.array(data)[:, plot_alts], axis=1)
    plt.plot(integrated, np.linspace(0.5, NUM_ALT - 0.5, NUM_ALT), 'k', linewidth=2)
    plt.xlim(-0.1, 1.1)
    plt.show()

    fig = plt.figure()
    ax1 = fig.add_subplot(111)
    ax2 = ax1.twiny()
    horiz = []
    stars = []
    fwhm = []
    for idx in range(0, NUM_ALT):
        data = A[LOC_IDX*NUM_ALT + idx, :].reshape((-1, NUM_ALT))
        raw_data = data.sum(axis=1)
        data = data.sum(axis=1)*6 + alts[idx] - np.max(data.sum(axis=1) * 6)
        horiz.append(data)
        stars.append(alts[idx])
        p0 = [LOC_IDX, 1]  # Inital guess is a normal distribution
        errfunc = lambda p, x, y: gauss(x, p) - y  # Distance to the target function
        p1, success = opt.leastsq(errfunc, p0[:], args=(np.arange(0, len(data)), raw_data))
        fit_mu, fit_stdev = p1
        FWHM = 2 * np.sqrt(2 * np.log(2)) * fit_stdev
        fwhm.append(FWHM)

    ax1.plot(LOC_IDX - np.arange(0, 159), np.array(horiz).T[:, plot_alts])

    for idx, dot in zip([LOC_IDX]*len(alts[plot_alts]), alts[plot_alts]):
        ax1.plot([0], [dot], '+', markersize=10)
    ax1.set_xlim(-5, 5)

    ax2.plot(np.array(fwhm[plot_alts]), alts[plot_alts], 'k--')
    ax2.set_xlim(0, 8)
    ax2.set_xlabel('FWHM [profiles]')

    plt.show()

def apriori_influence(A):
    ALT_IDX = 20


    NUM_ALT = 46
    plot_loc = slice(0, 159)

    plt.style.use('ggplot')

    alts = np.linspace(0.5, NUM_ALT - 0.5, NUM_ALT)
    plt.figure()
    # Vertical averaging kernels
    data = [np.sum(A[ALT_IDX::NUM_ALT, idx*(NUM_ALT):(idx+1)*NUM_ALT], axis=1) for idx in range(159)]
    plt.plot(data)
    plt.ylim(0, 1.5)

    integrated = np.sum(np.array(data), axis=1)
    plt.plot(integrated, 'k', linewidth=2)
    # plt.xlim(-0.1, 1.1)
    plt.show()


if __name__ == "__main__":
    # FILE = r'\\utls\\users\\data\\OMPS\\OMPS_Tomography_Testing\\v1.0\\ttt\\raw\\Retinfo_20538.h5'
    FILE = r'\\utls\\users\\data\\OMPS\\OMPS_Tomography_Testing\\v1.0\\new_triplet_40_40\\raw\\Retinfo_20538.h5'

    f = h5py.File(FILE, "r")

    A = f['RetInfo/Ozone_averaging_kernel'][:]
    make_averaging_kernel_plots(A)
    # apriori_influence(A)