import click
from tomography.omps_tomography import OMPSTomography
import tomography.omps_processing.omps_util as omps_util
import logging

@click.command()
@click.option('--l1g', multiple=True, type=click.Path(exists=True))
@click.option('--l1anc', multiple=True, type=click.Path(exists=True))
@click.option('--out', nargs=1, type=click.Path())
@click.option('--verbose', is_flag=True)
@click.option('--scatterorder', nargs=1, type=click.INT, default=30)
def retrieve(l1g, l1anc, out, verbose, scatterorder):
    import warnings
    warnings.filterwarnings("ignore")

    if verbose:
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    omps = OMPSTomography(0)
    omps.ret_opt['max_sza'] = 88
    omps.ret_opt['retrieval_order'] = ['albedo','aerosol','ozone','albedo','aerosol','ozone']
    omps.ret_opt['max_iter'] = [5, 5, 5, 2, 2, 2]
    omps.ret_opt['ozone_apriori_type'] = 'labow'
    omps.ret_opt['aerosol_apriori_type'] = 'default'
    omps.ret_opt['grid_type'] = 'scan'
    omps.ret_opt['tropopause_lowerbound'] = True
    omps.ret_opt['cloud_lowerbound'] = False
    omps.ret_opt['ozone_apriori_scaling'] = True
    omps.ret_opt['albedo_smooth_factor'] = 10
    omps.ret_opt['o3_horiz_tikh_factor'] = 100
    omps.ret_opt['o3_vertical_tikh_factor'] = 0.0
    omps.ret_opt['aero_horiz_tikh_factor'] = 100
    omps.ret_opt['aero_vertical_tikh_factor'] = 0.0
    omps.ret_opt['retrieval_method'] = 'rodgers'
    omps.ret_opt['ozone_lm_damping'] = 1.0
    omps.ret_opt['aerosol_lm_damping'] = 0.1

    omps.ret_opt['maxdifference_seconds'] = 100
    omps.ret_opt['scatterorder'] = scatterorder
    omps.ret_opt['num_diffuse_profiles'] = 5

    omps.ret_opt['l1g_input_files'] = [click.format_filename(f) for f in l1g]
    omps.ret_opt['l1_anc_input_files'] = [click.format_filename(f) for f in l1anc]

    output = omps.retrieve_orbit()

    omps_util.convert_pickle_to_l2(output, click.format_filename(out))

if __name__ == "__main__":
    retrieve()